import { useEffect, useState } from 'react';
import { Button } from 'react-bootstrap';
import ArchiveProducts from './ArchiveProducts.js';
import ActivateProducts from './ActivateProducts.js';
import SaveEdit from './SaveEdit.js';

const ProductRow = ({ product, fetchData }) => {
  const [originalValues, setOriginalValues] = useState({
    name: product.name,
    description: product.description,
    price: product.price,
    inventory: product.inventory,
  });

  const [selectedProductName, setSelectedProductName] = useState(product.name);
  const [selectedProductDescription, setSelectedProductDescription] = useState(product.description);
  const [selectedProductPrice, setSelectedProductPrice] = useState(product.price);
  const [selectedProductInventory, setSelectedProductInventory] = useState(product.inventory);
  const [selectedProductIsActive, setSelectedProductIsActive] = useState(product.isActive);
  const [isEditEnabled, setIsEditEnabled] = useState(false);
  const [isSaveEnabled, setIsSaveEnabled] = useState(false);

  const availabilityText = selectedProductIsActive ? 'Available' : 'Unavailable';
  const textColor = selectedProductIsActive ? 'text-success' : 'text-danger';
  const bgColor = selectedProductIsActive ? 'btn-archive' : 'btn-activate';

  useEffect(() => {
    // Update the local state when the product prop changes
    setSelectedProductName(product.name);
    setSelectedProductDescription(product.description);
    setSelectedProductPrice(product.price);
    setSelectedProductInventory(product.inventory);
    setSelectedProductIsActive(product.isActive);

    // Update the originalValues when the product prop changes
    setOriginalValues({
      name: product.name,
      description: product.description,
      price: product.price,
      inventory: product.inventory,
    });

    // Close edit if changes in data
    setIsEditEnabled(false); 

  }, [product]);

  const renderActionCell = (bgColor) => {
    if (selectedProductIsActive && selectedProductInventory !== 0) {
      return <td><ArchiveProducts productId={product._id} fetchData={fetchData} btnColor={bgColor} /></td>;
    } else if (!selectedProductIsActive && selectedProductInventory !== 0) {
      return <td><ActivateProducts productId={product._id} fetchData={fetchData} btnColor={bgColor} /></td>;
    } else if ((!selectedProductIsActive && selectedProductInventory === 0) || (selectedProductIsActive && selectedProductInventory === 0)) {
      return <td>Update product inventory to activate</td>;
    }
  };

  const renderProductCell = () => {
    if (product.image) {
      return (
        <>
          <span className="d-block bg-transparent">{selectedProductName}</span>
          <div className="d-block bg-transparent">
            <img src={`${process.env.REACT_APP_IMAGE_FOLDER}/${product.image.imageName}`} alt="" className="img-fluid" />
          </div>
        </>
      );
    } else {
      return selectedProductName;
    }
  };

  const enableEditMode = () => {
    setOriginalValues({
      name: selectedProductName,
      description: selectedProductDescription,
      price: selectedProductPrice,
      inventory: selectedProductInventory,
    });
    setIsEditEnabled(true);
  };

  const cancelEditMode = () => {
    // Reset the state values to the original values
    setSelectedProductName(originalValues.name);
    setSelectedProductDescription(originalValues.description);
    setSelectedProductPrice(originalValues.price);
    setSelectedProductInventory(originalValues.inventory);
    setIsEditEnabled(false);
  };

  const [editedValues, setEditedValues] = useState({
    id: product._id,
    name: selectedProductName,
    description: selectedProductDescription,
    price: selectedProductPrice,
    inventory: selectedProductInventory,
  });

  return (
    <>
      <td className="text-break">{product._id}</td>
      <td className="fw-bold">
        {isEditEnabled ? (
          <input
            type="text"
            name="productName"
            className="w-100"
            value={selectedProductName}
            onChange={(e) => {
              setSelectedProductName(e.target.value);
              setEditedValues((prevValues) => ({ ...prevValues, name: e.target.value }));
            }}
            disabled={!isEditEnabled}
          />
        ) : (
          renderProductCell()
        )}
      </td>
      <td>
        {isEditEnabled ? (
          <textarea
            name="productDescription"
            className="w-100 h-100"
            value={selectedProductDescription}
            onChange={(e) => {
              setSelectedProductDescription(e.target.value);
              setEditedValues((prevValues) => ({ ...prevValues, description: e.target.value }));
            }}
            disabled={!isEditEnabled}
          />
        ) : (
          selectedProductDescription
        )}
      </td>
      <td>
        {isEditEnabled ? (
          <input
            type="number"
            name="productPrice"
            className="w-100"
            value={selectedProductPrice}
            onChange={(e) => {
              setSelectedProductPrice(e.target.value);
              setEditedValues((prevValues) => ({ ...prevValues, price: e.target.value }));
            }}
            disabled={!isEditEnabled}
          />
        ) : (
          parseFloat(selectedProductPrice).toLocaleString('en-PH', { style: 'currency', currency: 'PHP' })
        )}
      </td>
      <td>
        {isEditEnabled ? (
          <input
            type="number"
            name="productInventory"
            value={selectedProductInventory}
            onChange={(e) => {
              setSelectedProductInventory(e.target.value);
              setEditedValues((prevValues) => ({ ...prevValues, inventory: e.target.value }));
            }}
            disabled={!isEditEnabled}
          />
        ) : (
          selectedProductInventory
        )}
      </td>
      <td className={`${textColor} fw-bold`}>{availabilityText}</td>

      {(!isEditEnabled) ?
        <td><Button onClick={() => { setIsEditEnabled(true) && setIsSaveEnabled(true) }} className="btn-submit nes-btn"><span className="far fa-edit"></span></Button></td>
        :
        <td><SaveEdit editedValues={editedValues} fetchData={fetchData} /></td>
      }
      {
        (!isEditEnabled) ? renderActionCell(bgColor) : <td><Button onClick={() => { setIsEditEnabled(false); cancelEditMode(); }} className="btn-archive nes-btn">Cancel Edit</Button></td>
      }
    </>
  );
};

export default ProductRow;
