import React, { useState, useEffect, useRef } from 'react';
import { Container, Table, Button } from 'react-bootstrap';

import Swal from 'sweetalert2';

const MyCart = () => {
  const [cartItems, setCartItems] = useState([]);
  const [selectedItems, setSelectedItems] = useState([]);
  const [quantities, setQuantities] = useState({}); // State to store quantities
  const checkboxesRef = useRef({}); // Ref to store checkboxes

  const [selectAll, setSelectAll] = useState(false); // State for "Select All" checkbox
  const [orderedItems, setOrderedItems] = useState([]); // State to store ordered items

  const fetchCartItems = async () => {
    try {
      const response = await fetch(`${process.env.REACT_APP_API_BASE_URL}/products/view-my-cart`, {
        method: "POST",
        headers: {
          'Authorization': `Bearer ${localStorage.token}`,
        },
      });

      if (response.ok) {
        const cartData = await response.json();
        setCartItems(cartData.cartItems);
        // Initialize quantities from cartItems
        const newQuantities = {};
        cartData.cartItems.forEach(item => {
          newQuantities[item.productId] = item.quantity || 0;
        });
        setQuantities(newQuantities);
      } else {
        console.error('Error fetching cart items:', response.statusText);
      }
    } catch (error) {
      console.error('Error fetching cart items:', error);
    }
  };

  useEffect(() => {

    fetchCartItems();

  }, [fetchCartItems]);

  const handleSelectAll = () => {
    const selectedIds = cartItems.map(item => item.productId);
    setSelectedItems(selectAll ? [] : selectedIds);
    setSelectAll(!selectAll);

    // Update orderedItems based on the selectedItems
      const updatedOrderedItems = cartItems.map(item => ({
        productId: item.productId,
        quantity: selectedItems.includes(item.productId) ? (quantities[item.productId] || 0) : 0
      }));
      setOrderedItems(updatedOrderedItems);
  };

  const handleDeselectAll = () => {
  setSelectedItems([]); // Deselect all items
  setSelectAll(false); // Update selectAll state

  setOrderedItems([]);
};


  const handleSelectItem = (productId) => {
    if (selectedItems.includes(productId)) {
      setSelectedItems(selectedItems.filter(id => id !== productId));
    } else {
      setSelectedItems([...selectedItems, productId]);
    }
  };

  const handleCheckboxChange = (productId) => {
    checkboxesRef.current[productId].checked = !checkboxesRef.current[productId].checked;
    handleSelectItem(productId);
    updateOrderedItems(productId, quantities[productId] || 0);
  };

  const handleQuantityChange = async (productId, value) => {
    try {
      // Prepare the request body
      const requestBody = JSON.stringify({
        productId: productId,
        quantity: value
      });

      // Send a POST request to update product quantity
      const response = await fetch(`${process.env.REACT_APP_API_BASE_URL}/products/update-quantity`, {
        method: 'POST',
        headers: {
          'Content-Type': 'application/json',
          'Authorization': `Bearer ${localStorage.token}`
        },
        body: requestBody
      });

      // Check if request was successful
      if (response.ok) {
        // If successful, update state with new quantity
        setQuantities({
          ...quantities,
          [productId]: Math.max(0, value) // Ensure quantity is non-negative
        });
        updateOrderedItems(productId, value);
      } else {
        // If not successful, handle error response
        const errorData = await response.json();
        console.error('Error updating product quantity:', errorData.message);
      }
    } catch (error) {
      // Handle network or other errors
      console.error('Error updating product quantity:', error);
    }
  };

const updateOrderedItems = (productId, quantity) => {
    const index = orderedItems.findIndex(item => item.productId === productId);
    if (index !== -1) {
      // If item exists, update quantity
      const updatedItems = [...orderedItems];
      updatedItems[index].quantity = quantity;
      setOrderedItems(updatedItems);
    } else {
      // If item doesn't exist, add it to orderedItems
      setOrderedItems([...orderedItems, { productId, quantity }]);
    }
  };

const handleRemoveFromCart = async () => {
  try {

    const productIds = JSON.stringify({ productIds: selectedItems });

    const response = await fetch(`${process.env.REACT_APP_API_BASE_URL}/products/remove`, {
      method: 'DELETE',
      headers: {
        'Content-Type': 'application/json',
        'Authorization': `Bearer ${localStorage.token}`
      },
      body: productIds
    });

    if (response.ok) {

      Swal.fire({
          title: "Product removed!",
          icon: "success",
          text: "Product/s removed from cart!"
        })

      fetchCartItems();

    } else {

      const errorData = await response.json();

      Swal.fire({
          title: "Remove from cart error!",
          icon: "error",
          text: "Error removing item/s from cart!"
        })

      console.error('Error removing products from cart:', errorData.message);
    }
  } catch (error) {
    // Handle network or other errors
    console.error('Error removing products from cart:', error);
  }
};

const handleCheckoutCartItems = async () => {

    const itemsTocheckOut = JSON.stringify({ orderedItems: orderedItems });

    fetch(`${process.env.REACT_APP_API_BASE_URL}/products/create-order`, {
      method : "POST",
      headers: {
        Authorization: `Bearer ${localStorage.getItem("token")}`,
        "Content-Type" : "application/json"
      },
      body: itemsTocheckOut
    }).then(result => result.json())
    .then(data => {

      if (data) {

        Swal.fire({
          title: "Order succesful!",
          icon: "success",
          text: "Product successfully ordered."
        })

        handleRemoveFromCart()
        
      }else{
        Swal.fire({
          title: "Something went wrong",
          icon: "error",
          text: "Please try again"
        })
      }
    })

}

  const calculateSubtotal = (item) => {
    return item.price * (quantities[item.productId] || 0);
  };

  // Calculate grand total for selected items
  const grandTotal = cartItems.reduce((total, item) => {
    if (selectedItems.includes(item.productId)) {
      return total + calculateSubtotal(item);
    }
    return total;
  }, 0);

  // Create a variable to contain the mapping of cart items
  const cartItemRows = cartItems.map((item, index) => (
    <tr key={item.id}>
      <td>
        <input
          type="checkbox"
          ref={ref => checkboxesRef.current[item.productId] = ref} // Store ref for each checkbox
          name="selectedItem"
          value={item.productId}
          checked={selectedItems.includes(item.productId)}
          onChange={() => handleCheckboxChange(item.productId)}
        />
      </td>
      <td>{index + 1}</td>
      <td>{item.name}</td>
      <td>{(item.price).toLocaleString('en-PH', { style: 'currency', currency: 'PHP' })}</td>
      <td className="d-flex">
        <Button className="nes-btn is-error" onClick={() => handleQuantityChange(item.productId, Math.max(0, (quantities[item.productId] || 0) - 1))}>-</Button>
        <span className="mx-auto my-auto">{quantities[item.productId]}</span>
        <Button className="nes-btn is-success" onClick={() => handleQuantityChange(item.productId, (quantities[item.productId] || 0) + 1)}>+</Button>
      </td>
      <td>{calculateSubtotal(item).toLocaleString('en-PH', { style: 'currency', currency: 'PHP' })}</td>
    </tr>
  ));

  return (
    <Container className="fs-4">
      <h2 className="mt-3 text-center headers mt-5">My Cart</h2>

      <Table striped bordered hover responsive className="mt-5">
        <thead>
          <tr>
            <th>
              <input type="checkbox" checked={selectAll} onChange={!selectAll ? handleSelectAll : handleDeselectAll} />
            </th>
            <th>#</th>
            <th>Name</th>
            <th>Price</th>
            <th className="text-center">Quantity</th>
            <th>Subtotal</th>
          </tr>
        </thead>
        <tbody>
          {cartItemRows}
        </tbody>
      </Table>
      <div className="d-flex justify-content-end">
        <p className="mt-3 fs-2">Grand Total: {grandTotal.toLocaleString('en-PH', { style: 'currency', currency: 'PHP' })}</p>
      </div>
      <div className="d-flex justify-content-end">
        <Button className="nes-btn mt-3 me-5" disabled={selectedItems.length === 0} onClick={handleCheckoutCartItems}>Proceed to Checkout</Button>
        <Button className="nes-btn is-error mt-3" disabled={selectedItems.length === 0} onClick={handleRemoveFromCart}>Remove from Cart</Button>
      </div>
    </Container>
  );
};

export default MyCart;
